Type: Package
Package: worldmet
Title: Import Surface Meteorological Data from NOAA
Version: 1.1.0
Authors@R: c(
    person("David", "Carslaw", , "david.carslaw@york.ac.uk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0991-950X")),
    person("Jack", "Davison", , "jack.davison@wsp.com", role = "aut",
           comment = c(ORCID = "0000-0003-2653-6615"))
  )
Description: Functions to import data from more than 30,000 surface
    meteorological sites around the world managed by the National Oceanic
    and Atmospheric Administration (NOAA) Global Historical Climate
    Network (GHCN) and Integrated Surface Database (ISD).
License: MIT + file LICENSE
URL: https://openair-project.github.io/worldmet/,
        https://github.com/openair-project/worldmet
BugReports: https://github.com/openair-project/worldmet/issues
Depends: R (>= 4.1.0)
Imports: carrier, cli, dplyr, lifecycle, mirai, purrr (>= 1.1.0),
        readr, rlang, sf, tibble, tidyr
Suggests: arrow, knitr, leaflet, rmarkdown
ByteCompile: true
Config/Needs/website: openair-project/openairpkgdown,openair
Encoding: UTF-8
Language: en-GB
LazyData: true
LazyLoad: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 07:31:49 UTC; JD38
Author: David Carslaw [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0991-950X>),
  Jack Davison [aut] (ORCID: <https://orcid.org/0000-0003-2653-6615>)
Maintainer: David Carslaw <david.carslaw@york.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-18 08:00:02 UTC
Built: R 4.6.0; ; 2026-05-18 09:27:30 UTC; unix
