#' Codes for weather types
#'
#' This data frame consists of the weather description codes used in the ISD. It
#' is not of general use to most users.
#'
#' \describe{
#'   \item{pwc}{Weather code, which can be merged with the `pwc` column in [importNOAA()] datasets.}
#'   \item{description}{Description associated with the weather codes.}
#' }
#' @examples
#' weatherCodes
"weatherCodes"
